-----------------------------------------------------------------------------------------------
-- Client Lua Script for GadgetBugFix
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "ActionSetLib"

local VERSIONMAJOR = 0
local VERSIONMINOR = 4

local GadgetBugFix = {} 
 
function GadgetBugFix:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function GadgetBugFix:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
function GadgetBugFix:OnLoad()
	Apollo.RegisterEventHandler("PlayerEquippedItemChanged", "OnEquippedItemChanged", self)
	Apollo.RegisterEventHandler("SystemKeyDown", "OnSystemKeyDown", self)
	Apollo.RegisterEventHandler("ChangeWorld", "ResetHTCC", self)
	Apollo.RegisterEventHandler("SpecChanged", "ResetHTCC", self)
	Apollo.RegisterEventHandler("CombatLogLAS", "ResetHTCC", self)
	Apollo.RegisterEventHandler("ActionSetError", "OnActionSetError", self)
	Apollo.RegisterSlashCommand("htcc", "OnHTCCCommand", self)
	ResetHTCC = self.ResetHTCC
	self.nLastFixAttempt = 0
	
	--[[local fnOldLoadForm = Apollo.LoadForm
	self.fnOldLoadForm = fnOldLoadForm
	Apollo.LoadForm = function (...)
		local a,b,c,d = ...
		if type(d) == "table" and d.wndTrackerLoot then
			_ChallengeTracker = d
		end
		return fnOldLoadForm(...)
	end
	--]]
	self.xmlDoc = XmlDoc.CreateFromFile("TargetFrameBuffBarFix.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)	
end

function GadgetBugFix:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		if Apollo.GetAddon("TargetFrame") then
			self:HookTargetFrameWindowCreation()
		else
			self.nTargetFrameFixAttemptCounter = 0
			self.timerTargetFrameFix = ApolloTimer.Create("3", true, "OnTargetFrameFixTimer", self)
		end
	end

end

function GadgetBugFix:OnHTCCCommand()
	self:ResetHTCC()
end

function GadgetBugFix:OnTargetFrameFixTimer()
	if Apollo.GetAddon("TargetFrame") then
		self:HookTargetFrameWindowCreation()
		if self.timerTargetFrameFix then
			self.timerTargetFrameFix:Stop()
			self.timerTargetFrameFix = nil
		end		
	elseif self.nTargetFrameFixAttemptCounter > 5 then
		if self.timerTargetFrameFix then
			self.timerTargetFrameFix:Stop()
			self.timerTargetFrameFix = nil
		end
	end
	self.nTargetFrameFixAttemptCounter = self.nTargetFrameFixAttemptCounter + 1
end

function GadgetBugFix:ResetHTCC()
	if Apollo.GetConsoleVariable("spell.holdToContinueCasting") then
		Apollo.SetConsoleVariable("spell.holdToContinueCasting", false)
		Apollo.SetConsoleVariable("spell.holdToContinueCasting", true)
	end
end

function GadgetBugFix:OnSystemKeyDown(nKey)
	if nKey == 27 then -- 27 = Escape
		self:ResetHTCC()
	end
end

function GadgetBugFix:OnEquippedItemChanged(eSlot, itemNew, itemOld)
	if eSlot == GameLib.CodeEnumEquippedItems.Gadget then
		local arCurrentActionSet = ActionSetLib.GetCurrentActionSet()
	
		local nSpellId = 0
		if itemNew then
			local tDetailedInfo = itemNew:GetDetailedInfo().tPrimary
			for idx, tSpell in ipairs(tDetailedInfo.arSpells) do
				if tSpell.bActivate then
					nSpellId = tSpell.splData:GetBaseSpellId()
				end
			end
		end
		--Print(arCurrentActionSet[9].." - "..nSpellId)
		
		if arCurrentActionSet[9] ~= nSpellId then
			--arCurrentActionSet[9] = 0
			--ActionSetLib.RequestActionSetChanges(arCurrentActionSet)
			arCurrentActionSet[9] = nSpellId
			ActionSetLib.RequestActionSetChanges(arCurrentActionSet)
		end
	end
end

function GadgetBugFix:OnActionSetError(eError)
	if eError == ActionSetLib.CodeEnumLimitedActionSetResult.BadSpellInActionSet then
		local arCurrentActionSet = ActionSetLib.GetCurrentActionSet()
		local itemGadget = Item.GetItemFromInventoryLoc(GameLib.CodeEnumEquippedItems.Gadget)
		local nSpellId = 0
		if itemGadget then
			local tDetailedInfo = itemGadget:GetDetailedInfo().tPrimary
			for idx, tSpell in ipairs(tDetailedInfo.arSpells) do
				if tSpell.bActivate then
					nSpellId = tSpell.splData:GetBaseSpellId()
				end
			end
		end
		arCurrentActionSet[9] = 0
		ActionSetLib.RequestActionSetChanges(arCurrentActionSet)
		arCurrentActionSet[9] = nSpellId
		self.arPendingActionSet = arCurrentActionSet
		if not self.timerChangeActionSet then
			self.timerChangeActionSet = ApolloTimer.Create("0.5", false, "OnActionSetTimer", self)
		else
			self.timerChangeActionSet:Stop()
			self.timerChangeActionSet:Start()
		end
		ActionSetLib.RequestActionSetChanges(arCurrentActionSet)
	end
end

function GadgetBugFix:OnActionSetTimer()
	if self.timerChangeActionSet then
		self.timerChangeActionSet:Stop()
		self.timerChangeActionSet = nil
	end
	if (Apollo.GetTickCount() - self.nLastFixAttempt) < 2000  then return end
	local arPathAbilities = AbilityBook.GetAbilitiesList(Spell.CodeEnumSpellTag.Path)
	local nPathSpellId = self.arPendingActionSet[10]
	local bSpellFound = false
	--Print(nPathSpellId)
	for idx, tSpell in pairs(arPathAbilities) do
		--Print(tSpell.nId)
		if tSpell.nId == nPathSpellId then bSpellFound = true end
	end
	if not bSpellFound then
		-- fallback for path oddities
		self.arPendingActionSet[9] = 0
		self.arPendingActionSet[10] = 0
	end
	self.nLastFixAttempt = Apollo.GetTickCount()
	ActionSetLib.RequestActionSetChanges(self.arPendingActionSet)
	self.arPendingActionSet = nil
	Print("Gadget bug fixed")
end

function GadgetBugFix:HookTargetFrameWindowCreation()
	local TargetFrame = Apollo.GetAddon("TargetFrame")
	
	local fnOld = TargetFrame.OnUnitFrameOptionsUpdated
	TargetFrame.OnUnitFrameOptionsUpdated = function(a)
		fnOld(a)
		Apollo.GetAddon("GadgetBugFix"):PerformTargetFrameFix()
	end
	
	if TargetFrame.luaUnitFrame then
		self:PerformTargetFrameFix()
	end
end

function GadgetBugFix:PerformTargetFrameFix()
	local TargetFrame = Apollo.GetAddon("TargetFrame")
	
	local luaFrames = {
		TargetFrame.luaFocusFrame,
		TargetFrame.luaTargetFrame,
		TargetFrame.luaUnitFrame,	
	}
	
	for idx, luaFrame in ipairs(luaFrames) do
		luaFrame.wndHarmBuffBar:Destroy()
		luaFrame.wndBeneBuffBar:Destroy()
		local wnd = Apollo.LoadForm(self.xmlDoc, "BuffForm", "FixedHudStratumLow", luaFrame)
		luaFrame.wndHarmBuffBar = wnd:FindChild("HarmBuffBar")
		luaFrame.wndBeneBuffBar = wnd:FindChild("BeneBuffBar")
		luaFrame.wndBuffBar = wnd
		local wndParent = luaFrame.wndLargeFrame:GetParent()
		local l,t,r,b = wndParent:GetAnchorPoints()
		wnd:SetAnchorPoints(l,t,r,b)
		local l,t,r,b = wndParent:GetAnchorOffsets()
		
		wnd:Show(wndParent:IsShown())
		if not luaFrame.tParams.bFlipped then
			wnd:SetAnchorOffsets(l+20,t-100,r-25,t-3)
			luaFrame.wndBeneBuffBar:SetStyleEx("AlignBuffsRight", true)
			luaFrame.wndHarmBuffBar:SetStyleEx("AlignBuffsRight", true)
		else
			wnd:SetAnchorOffsets(l+25,t-100,r-20,t-3)
		end
		luaFrame.__index.OnWindowMove = function(c,d,e)
			local luaFrame = c
			local wndParent = c.wndLargeFrame:GetParent()
			local l,t,r,b = wndParent:GetAnchorPoints()
			luaFrame.wndBuffBar:SetAnchorPoints(l,t,r,b)
			local l,t,r,b = wndParent:GetAnchorOffsets()
			luaFrame.wndBuffBar:SetAnchorOffsets(l+20,t-100,r-25,t-3)
		end
		
		luaFrame.__index.OnWindowShow = function(c,d,e)
			if d == e then
				local luaFrame = c
				luaFrame.wndBuffBar:Show(true)
			end
		end
		luaFrame.__index.OnWindowHide = function(c,d,e)
			if d == e then
				local luaFrame = c
				luaFrame.wndBuffBar:Show(false)
			end
		end
		wndParent:AddEventHandler("WindowMove", "OnWindowMove")
		wndParent:AddEventHandler("WindowShow", "OnWindowShow")
		wndParent:AddEventHandler("WindowHide", "OnWindowHide")
	end

end

local GadgetBugFixInst = GadgetBugFix:new()
GadgetBugFixInst:Init()
